@@ -19,6 +19,7 @@ from six import string_types
 from contextlib import contextmanager
 from collections import Counter
 from copy import deepcopy
+import logging
 
 # 3rd party
 import numpy as np
@@ -153,7 +154,7 @@ class AbstractObject(object):
 
         """
         content_dict = self._as_dict()
-        content_dict['tfields_version'] = tfields.__version__
+        content_dict["tfields_version"] = tfields.__version__
         np.savez(path, **content_dict)
 
     @classmethod
@@ -164,10 +165,10 @@ class AbstractObject(object):
         """
         # TODO: think about allow_pickle, wheter it really should be True or
         # wheter we could avoid pickling (potential security issue)
-        load_kwargs.setdefault('allow_pickle', True)
+        load_kwargs.setdefault("allow_pickle", True)
         np_file = np.load(path, **load_kwargs)
         d = dict(np_file)
-        d.pop('tfields_version', None)
+        d.pop("tfields_version", None)
         return cls._from_dict(d)
 
     def _args(self) -> tuple:
@@ -176,7 +177,7 @@ class AbstractObject(object):
     def _kwargs(self) -> dict:
         return dict()
 
-    _HIERARCHY_SEPARATOR = '::'
+    _HIERARCHY_SEPARATOR = "::"
 
     def _as_dict(self):
         d = {}
@@ -186,17 +187,15 @@ class AbstractObject(object):
 
         # args and kwargs
         for base_attr, iterable in [
-                ('args', ((str(i), arg)
-                          for i, arg in enumerate(self._args()))),
-                ('kwargs', self._kwargs().items())]:
+            ("args", ((str(i), arg) for i, arg in enumerate(self._args()))),
+            ("kwargs", self._kwargs().items()),
+        ]:
             for attr, value in iterable:
                 attr = base_attr + self._HIERARCHY_SEPARATOR + attr
-                if hasattr(value, '_as_dict'):
+                if hasattr(value, "_as_dict"):
                     part_dict = value._as_dict()
                     for part_attr, part_value in part_dict.items():
-                        d[
-                            attr + self._HIERARCHY_SEPARATOR + part_attr
-                        ] = part_value
+                        d[attr + self._HIERARCHY_SEPARATOR + part_attr] = part_value
                 else:
                     d[attr] = value
         return d
@@ -204,7 +203,7 @@ class AbstractObject(object):
     @classmethod
     def _from_dict(cls, d: dict):
         try:
-            d.pop('type')
+            d.pop("type")
         except KeyError:
             # legacy
             return cls._from_dict_legacy(**d)
@@ -226,7 +225,7 @@ class AbstractObject(object):
         """
         for attr in here:
             for key in here[attr]:
-                if 'type' in here[attr][key]:
+                if "type" in here[attr][key]:
                     obj_type = here[attr][key].get("type")
                     if isinstance(obj_type, np.ndarray):  # happens on np.load
                         obj_type = obj_type.tolist()
@@ -237,15 +236,15 @@ class AbstractObject(object):
                     obj_type = getattr(tfields, obj_type)
                     attr_value = obj_type._from_dict(here[attr][key])
                 else:  # if len(here[attr][key]) == 1:
-                    attr_value = here[attr][key].pop('')
+                    attr_value = here[attr][key].pop("")
                 here[attr][key] = attr_value
 
-        '''
+        """
         Build the generic way
-        '''
-        args = here.pop('args', tuple())
+        """
+        args = here.pop("args", tuple())
         args = tuple(args[key] for key in sorted(args))
-        kwargs = here.pop('kwargs', {})
+        kwargs = here.pop("kwargs", {})
         assert len(here) == 0
         obj = cls(*args, **kwargs)
         return obj
@@ -293,17 +292,17 @@ class AbstractObject(object):
                 bulk_type = getattr(tfields, bulk_type)
                 list_dict[key].append(bulk_type._from_dict_legacy(**sub_dict[index]))
 
-        with cls._bypass_setters('fields', demand_existence=False):
-            '''
+        with cls._bypass_setters("fields", demand_existence=False):
+            """
             Build the normal way
-            '''
-            bulk = kwargs.pop('bulk')
-            bulk_type = kwargs.pop('bulk_type')
+            """
+            bulk = kwargs.pop("bulk")
+            bulk_type = kwargs.pop("bulk_type")
             obj = cls.__new__(cls, bulk, **kwargs)
 
-            '''
+            """
             Set list attributes
-            '''
+            """
             for attr, list_value in list_dict.items():
                 setattr(obj, attr, list_value)
         return obj
@@ -373,9 +372,7 @@ class AbstractNdarray(np.ndarray, AbstractObject):
         slot_dtypes = cls.__slot_dtypes__ + [None] * (
             len(cls.__slots__) - len(cls.__slot_dtypes__)
         )
-        for attr, default, dtype in zip(
-            cls.__slots__, slot_defaults, slot_dtypes
-        ):
+        for attr, default, dtype in zip(cls.__slots__, slot_defaults, slot_dtypes):
             if attr == "_cache":
                 continue
             if attr not in kwargs:
@@ -459,7 +456,7 @@ class AbstractNdarray(np.ndarray, AbstractObject):
         """
         # Call the parent's __setstate__ with the other tuple elements.
         super(AbstractNdarray, self).__setstate__(
-            state[0:-len(self._iter_slots())]
+            state[0: -len(self._iter_slots())]
         )
 
         # set the __slot__ attributes
@@ -469,17 +466,18 @@ class AbstractNdarray(np.ndarray, AbstractObject):
         full information and thus need to be excluded from the __setstate__
         need to be in the same order as they have been added to __slots__
         """
-        added_slot_attrs = ['name']
+        added_slot_attrs = ["name"]
         n_np = 5  # number of numpy array states
         n_old = len(valid_slot_attrs) - len(state[n_np:])
         if n_old > 0:
             for latest_index in range(n_old):
                 new_slot = added_slot_attrs[-latest_index]
-                warnings.warn("Slots with names '{new_slot}' appears to have "
-                              "been added after the creation of the reduced "
-                              "state. No corresponding state found in "
-                              "__setstate__."
-                              .format(**locals()))
+                warnings.warn(
+                    "Slots with names '{new_slot}' appears to have "
+                    "been added after the creation of the reduced "
+                    "state. No corresponding state found in "
+                    "__setstate__.".format(**locals())
+                )
                 valid_slot_attrs.pop(valid_slot_attrs.index(new_slot))
                 setattr(self, new_slot, None)
 
@@ -497,9 +495,7 @@ class AbstractNdarray(np.ndarray, AbstractObject):
 
     @classmethod
     @contextmanager
-    def _bypass_setters(cls, *slots,
-                        empty_means_all=True,
-                        demand_existence=False):
+    def _bypass_setters(cls, *slots, empty_means_all=True, demand_existence=False):
         """
         Temporarily remove the setter in __slot_setters__ corresponding to slot
         position in __slot__. You should know what you do, when using this.
@@ -516,13 +512,11 @@ class AbstractNdarray(np.ndarray, AbstractObject):
         slot_indices = []
         setters = []
         for slot in slots:
-            slot_index = cls.__slots__.index(slot)\
-                if slot in cls.__slots__ else None
+            slot_index = cls.__slots__.index(slot) if slot in cls.__slots__ else None
             if slot_index is None:
                 # slot not in cls.__slots__.
                 if demand_existence:
-                    raise ValueError(
-                        "Slot {slot} not existing".format(**locals()))
+                    raise ValueError("Slot {slot} not existing".format(**locals()))
                 continue
             if len(cls.__slot_setters__) < slot_index + 1:
                 # no setter to be found
@@ -648,8 +642,9 @@ class Tensors(AbstractNdarray):
         Tensors([], shape=(0, 7), dtype=float64)
 
     """
-    __slots__ = ['coord_sys', 'name']
-    __slot_defaults__ = ['cartesian']
+
+    __slots__ = ["coord_sys", "name"]
+    __slot_defaults__ = ["cartesian"]
     __slot_setters__ = [tfields.bases.get_coord_system_name]
 
     def __new__(cls, tensors, **kwargs):
@@ -664,8 +659,8 @@ class Tensors(AbstractNdarray):
             coord_sys = kwargs.pop("coord_sys", tensors.coord_sys)
             tensors = tensors.copy()
             tensors.transform(coord_sys)
-            kwargs['coord_sys'] = coord_sys
-            kwargs['name'] = kwargs.pop('name', tensors.name)
+            kwargs["coord_sys"] = coord_sys
+            kwargs["name"] = kwargs.pop("name", tensors.name)
             if dtype is None:
                 dtype = tensors.dtype
         else:
@@ -680,8 +675,7 @@ class Tensors(AbstractNdarray):
             len(tensors)
         except TypeError:
             raise TypeError(
-                "Iterable structure necessary."
-                " Got {tensors}".format(**locals())
+                "Iterable structure necessary." " Got {tensors}".format(**locals())
             )
 
         """ process empty inputs """
@@ -693,12 +687,10 @@ class Tensors(AbstractNdarray):
             if issubclass(type(tensors), np.ndarray):
                 # np.empty
                 pass
-            elif hasattr(tensors, 'shape'):
+            elif hasattr(tensors, "shape"):
                 dim = dim(tensors)
             else:
-                raise ValueError(
-                    "Empty tensors need dimension parameter 'dim'."
-                )
+                raise ValueError("Empty tensors need dimension parameter 'dim'.")
 
         tensors = np.asarray(tensors, dtype=dtype, order=order)
         obj = tensors.view(cls)
@@ -830,14 +822,10 @@ class Tensors(AbstractNdarray):
                     pass
             if bases:
                 # get most frequent coord_sys
-                coord_sys = sorted(
-                    bases, key=Counter(bases).get, reverse=True
-                )[0]
+                coord_sys = sorted(bases, key=Counter(bases).get, reverse=True)[0]
                 kwargs["coord_sys"] = coord_sys
             else:
-                default = cls.__slot_defaults__[
-                    cls.__slots__.index("coord_sys")
-                ]
+                default = cls.__slot_defaults__[cls.__slots__.index("coord_sys")]
                 kwargs["coord_sys"] = default
 
         """ transform all raw inputs to cls type with correct coord_sys. Also
@@ -858,23 +846,24 @@ class Tensors(AbstractNdarray):
         for i, obj in enumerate(remainingObjects):
             tensors = np.append(tensors, obj, axis=0)
 
-        if len(tensors) == 0 and not kwargs.get('dim', None):
+        if len(tensors) == 0 and not kwargs.get("dim", None):
             # if you can not determine the tensor dimension, search for the
             # first object with some entries
-            kwargs['dim'] = dim(objects[0])
+            kwargs["dim"] = dim(objects[0])
 
         inst = cls.__new__(cls, tensors, **kwargs)
         if not return_templates:
             return inst
         else:
             tensor_lengths = [len(o) for o in objects]
-            cum_tensor_lengths = [sum(tensor_lengths[:i])
-                                  for i in range(len(objects))]
+            cum_tensor_lengths = [sum(tensor_lengths[:i]) for i in range(len(objects))]
             templates = [
                 tfields.TensorFields(
                     np.empty((len(obj), 0)),
-                    np.arange(tensor_lengths[i]) + cum_tensor_lengths[i])
-                for i, obj in enumerate(objects)]
+                    np.arange(tensor_lengths[i]) + cum_tensor_lengths[i],
+                )
+                for i, obj in enumerate(objects)
+            ]
             return inst, templates
 
     @classmethod
@@ -954,9 +943,7 @@ class Tensors(AbstractNdarray):
 
         """
         cls_kwargs = {
-            attr: kwargs.pop(attr)
-            for attr in list(kwargs)
-            if attr in cls.__slots__
+            attr: kwargs.pop(attr) for attr in list(kwargs) if attr in cls.__slots__
         }
         inst = cls.__new__(
             cls, tfields.lib.grid.igrid(*base_vectors, **kwargs), **cls_kwargs
@@ -1176,9 +1163,7 @@ class Tensors(AbstractNdarray):
                 + segment * periodicity / num_segments
             )
 
-    def equal(
-        self, other, rtol=None, atol=None, equal_nan=False, return_bool=True
-    ):
+    def equal(self, other, rtol=None, atol=None, equal_nan=False, return_bool=True):
         """
         Evaluate, whether the instance has the same content as other.
 
@@ -1189,10 +1174,7 @@ class Tensors(AbstractNdarray):
                 equal_nan (bool)
             see numpy.isclose
         """
-        if (
-            issubclass(type(other), Tensors)
-            and self.coord_sys != other.coord_sys
-        ):
+        if issubclass(type(other), Tensors) and self.coord_sys != other.coord_sys:
             other = other.copy()
             other.transform(self.coord_sys)
         x, y = np.asarray(self), np.asarray(other)
@@ -1418,13 +1400,13 @@ find-ordered-vector-in-numpy-array
         if template.fields and issubclass(type(self), TensorFields):
             template_field = np.array(template.fields[0])
             if len(self) > 0:
-                '''
+                """
                 if new vertices have been created in the template, it is
                 in principle unclear what fields we have to refer to.
                 Thus in creating the template, we gave np.nan.
                 To make it fast, we replace nan with 0 as a dummy and correct
                 the field entries afterwards with np.nan.
-                '''
+                """
                 nan_mask = np.isnan(template_field)
                 template_field[nan_mask] = 0  # dummy reference to index 0.
                 template_field = template_field.astype(int)
@@ -1526,10 +1508,7 @@ find-ordered-vector-in-numpy-array
             True
 
         """
-        if (
-            issubclass(type(other), Tensors)
-            and self.coord_sys != other.coord_sys
-        ):
+        if issubclass(type(other), Tensors) and self.coord_sys != other.coord_sys:
             other = other.copy()
             other.transform(self.coord_sys)
         return sp.spatial.distance.cdist(self, other, **kwargs)
@@ -1782,10 +1761,8 @@ class TensorFields(Tensors):
 
     """
 
-    __slots__ = ['coord_sys', 'name', 'fields']
-    __slot_setters__ = [tfields.bases.get_coord_system_name,
-                        None,
-                        as_tensors_list]
+    __slots__ = ["coord_sys", "name", "fields"]
+    __slot_setters__ = [tfields.bases.get_coord_system_name, None, as_tensors_list]
 
     def __new__(cls, tensors, *fields, **kwargs):
         rigid = kwargs.pop("rigid", True)
@@ -1805,9 +1782,7 @@ class TensorFields(Tensors):
             if not all([flen == olen for flen in field_lengths]):
                 raise ValueError(
                     "Length of base ({olen}) should be the same as"
-                    " the length of all fields ({field_lengths}).".format(
-                        **locals()
-                    )
+                    " the length of all fields ({field_lengths}).".format(**locals())
                 )
         return obj
 
@@ -1816,7 +1791,7 @@ class TensorFields(Tensors):
 
     def _kwargs(self):
         d = super()._kwargs()
-        d.pop('fields')
+        d.pop("fields")
         return d
 
     def __getitem__(self, index):
@@ -1864,8 +1839,7 @@ class TensorFields(Tensors):
                     index = index[0]
                 if item.fields:
                     # circumvent the setter here.
-                    with self._bypass_setters('fields',
-                                              demand_existence=False):
+                    with self._bypass_setters("fields", demand_existence=False):
                         item.fields = [
                             field.__getitem__(index) for field in item.fields
                         ]
@@ -1923,7 +1897,7 @@ class TensorFields(Tensors):
             )
 
         return_value = super(TensorFields, cls).merged(*objects, **kwargs)
-        return_templates = kwargs.get('return_templates', False)
+        return_templates = kwargs.get("return_templates", False)
         if return_templates:
             inst, templates = return_value
         else:
@@ -1963,8 +1937,11 @@ class TensorFields(Tensors):
     @names.setter
     def names(self, names):
         if not len(names) == len(self.fields):
-            raise ValueError("len(names) ({0}) != len(fields) ({1})"
-                             .format(len(names), len(self.fields)))
+            raise ValueError(
+                "len(names) ({0}) != len(fields) ({1})".format(
+                    len(names), len(self.fields)
+                )
+            )
         for i, name in enumerate(names):
             self.fields[i].name = name
 
@@ -2016,10 +1993,7 @@ class TensorFields(Tensors):
             if self.dim == field.dim:
                 field.transform(self.coord_sys)
             else:
-                import logging
-
-                log = logging.getLogger()
-                log.debug(
+                logging.debug(
                     "Careful: Plotting tensors with field of"
                     "different dimension. No coord_sys check performed."
                 )
@@ -2083,7 +2057,7 @@ class Container(Fields):
             self.append(item)
 
     def _kwargs(self):
-        return {'labels': self.labels}
+        return {"labels": self.labels}
 
 
 class Maps(sortedcontainers.SortedDict, AbstractObject):
@@ -2100,6 +2074,7 @@ class Maps(sortedcontainers.SortedDict, AbstractObject):
         )
 
     """
+
     def __init__(self, *args, **kwargs):
         if args and args[0] is None:
             # None key passed e.g. by copy. We do not change keys here.
@@ -2152,7 +2127,7 @@ class Maps(sortedcontainers.SortedDict, AbstractObject):
             else:
                 copy = True
         if copy:  # not else, because in case of wrong mp type we initialize
-            kwargs.setdefault('dtype', int)
+            kwargs.setdefault("dtype", int)
             mp = TensorFields(mp, *fields, **kwargs)
         return mp
 
@@ -2210,12 +2185,14 @@ class TensorMaps(TensorFields):
         >>> assert mesh_cp_cyl.coord_sys == tfields.bases.CYLINDER
 
     """
-    __slots__ = ['coord_sys', 'name', 'fields', 'maps']
-    __slot_setters__ = [tfields.bases.get_coord_system_name,
-                        None,
-                        as_tensors_list,
-                        as_maps,
-                        ]
+
+    __slots__ = ["coord_sys", "name", "fields", "maps"]
+    __slot_setters__ = [
+        tfields.bases.get_coord_system_name,
+        None,
+        as_tensors_list,
+        as_maps,
+    ]
 
     def __new__(cls, tensors, *fields, **kwargs):
         if issubclass(type(tensors), TensorMaps):
@@ -2302,15 +2279,13 @@ class TensorMaps(TensorFields):
                     map_mask = ~map_delete_mask
 
                     # build the correction counters
-                    move_up_counter = np.zeros(
-                        self.maps[map_dim].shape, dtype=int
-                    )
+                    move_up_counter = np.zeros(self.maps[map_dim].shape, dtype=int)
                     for p in delete_indices:
                         move_up_counter[self.maps[map_dim] > p] -= 1
 
-                    item.maps[map_dim] = (
-                        self.maps[map_dim] + move_up_counter
-                    )[map_mask]
+                    item.maps[map_dim] = (self.maps[map_dim] + move_up_counter)[
+                        map_mask
+                    ]
 
         return item
 
@@ -2319,16 +2294,13 @@ class TensorMaps(TensorFields):
         if not all([isinstance(o, cls) for o in objects]):
             # TODO: could allow if all face_fields are none
             raise TypeError(
-                "Merge constructor only accepts {cls} instances.".format(
-                    **locals()
-                )
+                "Merge constructor only accepts {cls} instances.".format(**locals())
             )
         tensor_lengths = [len(o) for o in objects]
-        cum_tensor_lengths = [sum(tensor_lengths[:i])
-                              for i in range(len(objects))]
+        cum_tensor_lengths = [sum(tensor_lengths[:i]) for i in range(len(objects))]
 
         return_value = super().merged(*objects, **kwargs)
-        return_templates = kwargs.get('return_templates', False)
+        return_templates = kwargs.get("return_templates", False)
         if return_templates:
             inst, templates = return_value
         else:
@@ -2346,18 +2318,17 @@ class TensorMaps(TensorFields):
         template_maps_list = [[] for i in range(len(objects))]
         for dimension in sorted(dim_maps_dict):
             # sort by object index
-            dim_maps = [dim_maps_dict[dimension][i]
-                        for i in range(len(objects))]
+            dim_maps = [dim_maps_dict[dimension][i] for i in range(len(objects))]
 
             return_value = TensorFields.merged(
-                *dim_maps,
-                return_templates=return_templates,
+                *dim_maps, return_templates=return_templates,
             )
             if return_templates:
                 mp, dimension_map_templates = return_value
                 for i in range(len(objects)):
-                    template_maps_list[i].append((dimension,
-                                                  dimension_map_templates[i]))
+                    template_maps_list[i].append(
+                        (dimension, dimension_map_templates[i])
+                    )
             else:
                 mp = return_value
             maps.append(mp)
@@ -2368,8 +2339,8 @@ class TensorMaps(TensorFields):
                 # template maps will not have dimensions according to their
                 # tensors which are indices
                 templates[i] = tfields.TensorMaps(
-                    templates[i],
-                    maps=Maps(template_maps))
+                    templates[i], maps=Maps(template_maps)
+                )
             return inst, templates
         else:
             return inst
@@ -2509,30 +2480,36 @@ class TensorMaps(TensorFields):
             copy of self without stale vertices and duplicat points (depending
             on arguments)
         """
-        # remove stale vertices
-        if stale:
-            stale_mask = self.stale()
-        else:
-            stale_mask = np.full(self.shape[0], False, dtype=bool)
-        # remove duplicates in order to not have any artificial separations
-        inst = self
-        if duplicates:
+        if not stale and not duplicates:
             inst = self.copy()
-            duplicates = tfields.duplicates(self, axis=0)
-            for tensor_index, duplicate_index in zip(
-                range(self.shape[0]), duplicates
-            ):
+        if stale:
+            # remove stale vertices i.e. those that are not referred by any
+            # map
+            remove_mask = self.stale()
+            inst = self.removed(remove_mask)
+        if duplicates:
+            # remove duplicates in order to not have any artificial separations
+            if not stale:
+                # we have not yet made a copy but want to work on inst
+                inst = self.copy()
+            remove_mask = np.full(inst.shape[0], False, dtype=bool)
+            duplicates = tfields.lib.util.duplicates(inst, axis=0)
+            for tensor_index, duplicate_index in zip(range(inst.shape[0]), duplicates):
                 if duplicate_index != tensor_index:
-                    stale_mask[tensor_index] = True
-                    # redirect maps
-                    for map_dim in self.maps:
-                        for f in range(len(self.maps[map_dim])):  # face index
-                            mp = np.array(self.maps[map_dim], dtype=int)
+                    # mark duplicate at tensor_index for removal
+                    remove_mask[tensor_index] = True
+                    # redirect maps. Note: work on inst.maps instead of
+                    # self.maps in case stale vertices where removed
+                    for map_dim in inst.maps:
+                        for f in range(len(inst.maps[map_dim])):  # face index
+                            mp = np.array(inst.maps[map_dim], dtype=int)
                             if tensor_index in mp[f]:
-                                index = tfields.index(mp[f], tensor_index)
+                                index = tfields.lib.util.index(mp[f], tensor_index)
                                 inst.maps[map_dim][f][index] = duplicate_index
-
-        return inst.removed(stale_mask)
+            if remove_mask.any():
+                # prevent another copy
+                inst = inst.removed(remove_mask)
+        return inst
 
     def removed(self, remove_condition):
         """
@@ -2605,7 +2582,6 @@ class TensorMaps(TensorFields):
             List(cls): One TensorMaps or TensorMaps subclass per
                 map_description
         """
-        # raise ValueError(map_descriptions)
         parts = []
         for map_description in map_descriptions:
             map_dim, map_indices_list = map_description
@@ -2740,8 +2716,7 @@ class TensorMaps(TensorFields):
         sorted_paths = []
         for path in paths:
             # find start index
-            values, counts = np.unique(path.maps[map_dim].flat,
-                                       return_counts=True)
+            values, counts = np.unique(path.maps[map_dim].flat, return_counts=True)
 
             first_node = None
             for v, c in zip(values, counts):
@@ -2790,4 +2765,5 @@ class TensorMaps(TensorFields):
 
 if __name__ == "__main__":  # pragma: no cover
     import doctest
+
     doctest.testmod()
