@@ -7,7 +7,6 @@ Mail:       daniel.boeckenhoff@ipp.mpg.de
 part of tfields library
 """
 import numpy as np
-import sympy
 import rna
 import tfields
 
@@ -18,7 +17,7 @@ import os
 
 
 def _dist_from_plane(point, plane):
-    return plane['normal'].dot(point) + plane['d']
+    return plane["normal"].dot(point) + plane["d"]
 
 
 def _segment_plane_intersection(p0, p1, plane):
@@ -92,41 +91,46 @@ def _intersect(triangle, plane, vertices_rejected):
     s2, d2 = _segment_plane_intersection(triangle[2], triangle[0], plane)
 
     single_index = index
-    couple_indices = [j for j in range(3)
-                      if not vertices_rejected[j] == lonely_bool]
+    couple_indices = [j for j in range(3) if not vertices_rejected[j] == lonely_bool]
 
     # TODO handle special cases. For now triangles with at least two points on plane are excluded
-    new_points = None
+    new_vertices = None
 
     if len(s0) == 2:
         # both points on plane
-        return new_points
+        return new_vertices
     if len(s1) == 2:
         # both points on plane
-        return new_points
+        return new_vertices
     if len(s2) == 2:
         # both points on plane
-        return new_points
+        return new_vertices
     if lonely_bool:
         # two new triangles
         if len(s0) == 1 and len(s1) == 1:
-            new_points = [[couple_indices[0], s0[0], couple_indices[1]],
-                          [couple_indices[1], complex(1), s1[0]]]
+            new_vertices = [
+                [couple_indices[0], s0[0], couple_indices[1]],
+                [couple_indices[1], complex(1), s1[0]],
+            ]
         elif len(s1) == 1 and len(s2) == 1:
-            new_points = [[couple_indices[0], couple_indices[1], s1[0]],
-                          [couple_indices[0], complex(2), s2[0]]]
+            new_vertices = [
+                [couple_indices[0], couple_indices[1], s1[0]],
+                [couple_indices[0], complex(2), s2[0]],
+            ]
         elif len(s0) == 1 and len(s2) == 1:
-            new_points = [[couple_indices[0], couple_indices[1], s0[0]],
-                          [couple_indices[1], s2[0], complex(2)]]
+            new_vertices = [
+                [couple_indices[0], couple_indices[1], s0[0]],
+                [couple_indices[1], s2[0], complex(2)],
+            ]
     else:
         # one new triangle
         if len(s0) == 1 and len(s1) == 1:
-            new_points = [[single_index, s1[0], s0[0]]]
+            new_vertices = [[single_index, s1[0], s0[0]]]
         elif len(s1) == 1 and len(s2) == 1:
-            new_points = [[single_index, s2[0], s1[0]]]
+            new_vertices = [[single_index, s2[0], s1[0]]]
         elif len(s0) == 1 and len(s2) == 1:
-            new_points = [[single_index, s0[0], s2[0]]]
-    return new_points
+            new_vertices = [[single_index, s0[0], s2[0]]]
+    return new_vertices
 
 
 class Mesh3D(tfields.TensorMaps):
@@ -191,19 +195,20 @@ class Mesh3D(tfields.TensorMaps):
         >>> assert np.array_equal(m1.faces, np.array([[0, 1, 2]]))
 
     """
+
     def __new__(cls, tensors, *fields, **kwargs):
-        kwargs['dim'] = 3
-        if 'maps' in kwargs and 'faces' in kwargs:
+        kwargs["dim"] = 3
+        if "maps" in kwargs and "faces" in kwargs:
             raise ValueError("Conflicting options maps and faces")
-        faces = kwargs.pop('faces', None)
-        maps = kwargs.pop('maps', None)
-        if faces is not None and not faces:
-            # faces = []
-            faces = np.empty((0, 3))
+        faces = kwargs.pop("faces", None)
+        maps = kwargs.pop("maps", None)
         if faces is not None:
+            if len(faces) == 0:
+                # faces = []
+                faces = np.empty((0, 3))
             maps = [faces]
         if maps is not None:
-            kwargs['maps'] = maps
+            kwargs["maps"] = maps
         obj = super(Mesh3D, cls).__new__(cls, tensors, *fields, **kwargs)
         if len(obj.maps) > 1:
             raise ValueError("Mesh3D only allows one map")
@@ -215,28 +220,29 @@ class Mesh3D(tfields.TensorMaps):
         """
         Save obj as wavefront/.obj file
         """
-        obj = kwargs.pop('object', None)
-        group = kwargs.pop('group', None)
+        obj = kwargs.pop("object", None)
+        group = kwargs.pop("group", None)
 
-        cmap = kwargs.pop('cmap', 'viridis')
-        map_index = kwargs.pop('map_index', None)
+        cmap = kwargs.pop("cmap", "viridis")
+        map_index = kwargs.pop("map_index", None)
 
-        path = path.replace('.obj', '')
+        path = path.replace(".obj", "")
         directory, name = os.path.split(path)
 
         if map_index is not None:
             scalars = self.maps[3].fields[map_index]
             min_scalar = scalars[~np.isnan(scalars)].min()
             max_scalar = scalars[~np.isnan(scalars)].max()
-            vmin = kwargs.pop('vmin', min_scalar)
-            vmax = kwargs.pop('vmax', max_scalar)
+            vmin = kwargs.pop("vmin", min_scalar)
+            vmax = kwargs.pop("vmax", max_scalar)
             if vmin == vmax:
-                if vmin == 0.:
-                    vmax = 1.
+                if vmin == 0.0:
+                    vmax = 1.0
                 else:
-                    vmin = 0.
+                    vmin = 0.0
             import matplotlib.colors as colors
             import matplotlib.pyplot as plt
+
             norm = colors.Normalize(vmin, vmax)
             color_map = plt.get_cmap(cmap)
         else:
@@ -248,7 +254,7 @@ class Mesh3D(tfields.TensorMaps):
             raise ValueError("Unused arguments.")
 
         if norm is not None:
-            mat_name = name + '_frame_{0}.mat'.format(map_index)
+            mat_name = name + "_frame_{0}.mat".format(map_index)
             scalars[np.isnan(scalars)] = min_scalar - 1
             sorted_scalars = scalars[scalars.argsort()]
             sorted_scalars[sorted_scalars == min_scalar - 1] = np.nan
@@ -256,20 +262,21 @@ class Mesh3D(tfields.TensorMaps):
             scalar_set = np.unique(sorted_scalars)
             scalar_set[scalar_set == min_scalar - 1] = np.nan
             mat_path = os.path.join(directory, mat_name)
-            with open(mat_path, 'w') as mf:
+            with open(mat_path, "w") as mf:
                 for s in scalar_set:
                     if np.isnan(s):
                         mf.write("newmtl nan")
                         mf.write("Kd 0 0 0\n\n")
                     else:
                         mf.write("newmtl mtl_{0}\n".format(s))
-                        mf.write("Kd {c[0]} {c[1]} {c[2]}\n\n"
-                                 .format(c=color_map(norm(s))))
+                        mf.write(
+                            "Kd {c[0]} {c[1]} {c[2]}\n\n".format(c=color_map(norm(s)))
+                        )
         else:
             sorted_faces = self.faces
 
         # writing of the obj file
-        with open(path + '.obj', 'w') as f:
+        with open(path + ".obj", "w") as f:
             f.write("# File saved with tfields Mesh3D._save_obj method\n\n")
             if norm is not None:
                 f.write("mtllib ./{0}\n\n".format(mat_name))
@@ -303,48 +310,49 @@ class Mesh3D(tfields.TensorMaps):
         Given a path to a obj/wavefront file, construct the object
         """
         import csv
+
         log = logging.getLogger()
 
-        with open(path, mode='r') as f:
-            reader = csv.reader(f, delimiter=' ')
+        with open(path, mode="r") as f:
+            reader = csv.reader(f, delimiter=" ")
             groups = []
             group = None
             vertex_no = 1
             for line in reader:
                 if not line:
                     continue
-                if line[0] == '#':
+                if line[0] == "#":
                     continue
-                if line[0] == 'g':
+                if line[0] == "g":
                     if group:
                         groups.append(group)
                     group = dict(name=line[1], vertices={}, faces=[])
-                elif line[0] == 'v':
+                elif line[0] == "v":
                     if not group:
                         log.debug("No group found. Setting default 'Group'")
-                        group = dict(name='Group', vertices={}, faces=[])
+                        group = dict(name="Group", vertices={}, faces=[])
                     vertex = list(map(float, line[1:4]))
-                    group['vertices'][vertex_no] = vertex
+                    group["vertices"][vertex_no] = vertex
                     vertex_no += 1
-                elif line[0] == 'f':
+                elif line[0] == "f":
                     face = []
                     for v in line[1:]:
-                        w = v.split('/')
+                        w = v.split("/")
                         face.append(int(w[0]))
-                    group['faces'].append(face)
+                    group["faces"].append(face)
             else:
                 groups.append(group)
 
         vertices = []
         for g in groups[:]:
-            vertices.extend(g['vertices'].values())
+            vertices.extend(g["vertices"].values())
 
         if len(group_names) != 0:
-            groups = [g for g in groups if g['name'] in group_names]
+            groups = [g for g in groups if g["name"] in group_names]
 
         faces = []
         for g in groups:
-            faces.extend(g['faces'])
+            faces.extend(g["faces"])
         faces = np.add(np.array(faces), -1).tolist()
 
         """
@@ -358,8 +366,10 @@ class Mesh3D(tfields.TensorMaps):
             if length == 3:
                 continue
             if length == 4:
-                log.warning("Given a Rectangle. I will split it but "
-                            "sometimes the order is different.")
+                log.warning(
+                    "Given a Rectangle. I will split it but "
+                    "sometimes the order is different."
+                )
                 faces.insert(i + 1, faces[i][2:] + faces[i][:1])
                 faces[i] = faces[i][:3]
             else:
@@ -380,8 +390,8 @@ class Mesh3D(tfields.TensorMaps):
         """
         vertices = tfields.Tensors.grid(*base_vectors, **kwargs)
 
-        base_vectors = tfields.grid.ensure_complex(*base_vectors)
-        base_vectors = tfields.grid.to_base_vectors(*base_vectors)
+        base_vectors = tfields.lib.grid.ensure_complex(*base_vectors)
+        base_vectors = tfields.lib.grid.to_base_vectors(*base_vectors)
         fix_coord = None
         for coord in range(3):
             if len(base_vectors[coord]) > 1:
@@ -446,8 +456,8 @@ class Mesh3D(tfields.TensorMaps):
         if not len(base_vectors) == 3:
             raise AttributeError("3 base_vectors vectors required")
 
-        base_vectors = tfields.grid.ensure_complex(*base_vectors)
-        base_vectors = tfields.grid.to_base_vectors(*base_vectors)
+        base_vectors = tfields.lib.grid.ensure_complex(*base_vectors)
+        base_vectors = tfields.lib.grid.to_base_vectors(*base_vectors)
 
         indices = [0, -1]
         coords = range(3)
@@ -465,8 +475,7 @@ class Mesh3D(tfields.TensorMaps):
         for ind in indices:
             for coord in coords:
                 basePart = base_vectors[:]
-                basePart[coord] = np.array([base_vectors[coord][ind]],
-                                           dtype=float)
+                basePart[coord] = np.array([base_vectors[coord][ind]], dtype=float)
                 planes.append(cls.plane(*basePart, **kwargs))
         inst = cls.merged(*planes, **kwargs)
         return inst
@@ -476,7 +485,10 @@ class Mesh3D(tfields.TensorMaps):
         if self.maps:
             return self.maps[3]
         else:
-            raise AttributeError("No faces found. Bug - please report")
+            logging.warning(
+                "No faces found. Mesh has {x} vertices.".format(x=len(self))
+            )
+            return tfields.Maps.to_map([], dim=3)
 
     @faces.setter
     def faces(self, faces):
@@ -531,16 +543,15 @@ class Mesh3D(tfields.TensorMaps):
         self.tree or setting it to self.tree = <saved tree> before
         calling in_faces
         """
-        key = 'mesh_tree'
-        if hasattr(self, '_cache') and key in self._cache:
+        key = "mesh_tree"
+        if hasattr(self, "_cache") and key in self._cache:
             log = logging.getLogger()
-            log.info(
-                "Using cached decision tree to speed up point - face mapping.")
-            masks = self.tree.in_faces(
-                points, delta, assign_multiple=assign_multiple)
+            log.info("Using cached decision tree to speed up point - face mapping.")
+            masks = self.tree.in_faces(points, delta, assign_multiple=assign_multiple)
         else:
             masks = self.triangles().in_triangles(
-                points, delta, assign_multiple=assign_multiple)
+                points, delta, assign_multiple=assign_multiple
+            )
         return masks
 
     @property
@@ -550,17 +561,18 @@ class Mesh3D(tfields.TensorMaps):
         hugely optimize 'in_faces' searches
 
         Examples:
-            >>> mesh = tfields.Mesh3D.grid((0, 1, 3), (1, 2, 3), (2, 3, 3))
-            >>> _ = mesh.tree
-            >>> assert hasattr(mesh, '_cache')
-            >>> assert 'mesh_tree' in mesh._cache
-            >>> mask = mesh.in_faces(tfields.Points3D([[0.2, 1.2, 2.0]]),
-            ...                      0.00001)
-            >>> assert mask.sum() == 1  # one point in one triangle
+            # >>> mesh = tfields.Mesh3D.grid((0, 1, 3), (1, 2, 3), (2, 3, 3))
+            # >>> _ = mesh.tree
+            # >>> assert hasattr(mesh, '_cache')
+            # >>> assert 'mesh_tree' in mesh._cache
+            # >>> mask = mesh.in_faces(tfields.Points3D([[0.2, 1.2, 2.0]]),
+            # ...                      0.00001)
+            # >>> assert mask.sum() == 1  # one point in one triangle
         """
-        if not hasattr(self, '_cache'):
+        raise ValueError("Broken feature. We are working on it!")
+        if not hasattr(self, "_cache"):
             self._cache = {}
-        key = 'mesh_tree'
+        key = "mesh_tree"
         if key in self._cache:
             tree = self._cache[key]
         else:
@@ -570,9 +582,9 @@ class Mesh3D(tfields.TensorMaps):
 
     @tree.setter
     def tree(self, tree):
-        if not hasattr(self, '_cache'):
+        if not hasattr(self, "_cache"):
             self._cache = {}
-        key = 'mesh_tree'
+        key = "mesh_tree"
         self._cache[key] = tree
 
     def remove_faces(self, face_delete_mask):
@@ -613,17 +625,19 @@ class Mesh3D(tfields.TensorMaps):
                 scalars.append(face_indices[face_mask][0])
             inst.maps[3].fields = [tfields.Tensors(scalars, dim=1)]
         else:
-            inst.maps = [tfields.TensorFields([],
-                                              tfields.Tensors([], dim=1),
-                                              dim=3,
-                                              dtype=int)
-                         ]
+            inst.maps = [
+                tfields.TensorFields([], tfields.Tensors([], dim=1), dim=3, dtype=int)
+            ]
         return inst
 
-    def project(self, tensor_field,
-                delta=None, merge_functions=None,
-                point_face_assignment=None,
-                return_point_face_assignment=False):
+    def project(
+        self,
+        tensor_field,
+        delta=None,
+        merge_functions=None,
+        point_face_assignment=None,
+        return_point_face_assignment=False,
+    ):
         """
         project the points of the tensor_field to a copy of the mesh
         and set the face values accord to the field to the maps field.
@@ -695,7 +709,7 @@ class Mesh3D(tfields.TensorMaps):
         # setup empty map fields and collect fields
         n_faces = len(self.maps[3])
         point_indices = np.arange(len(tensor_field))
-        if not hasattr(tensor_field, 'fields') or len(tensor_field.fields) == 0:
+        if not hasattr(tensor_field, "fields") or len(tensor_field.fields) == 0:
             # if not fields is existing use int type fields and empty_map_fields
             # in order to generate a sum
             fields = [np.full(len(tensor_field), 1, dtype=int)]
@@ -709,8 +723,7 @@ class Mesh3D(tfields.TensorMaps):
                 cls = type(field)
                 kwargs = {key: getattr(field, key) for key in cls.__slots__}
                 shape = (n_faces,) + field.shape[1:]
-                empty_map_fields.append(cls(np.full(shape, np.nan),
-                                            **kwargs))
+                empty_map_fields.append(cls(np.full(shape, np.nan), **kwargs))
             if merge_functions is None:
                 merge_functions = [lambda x: np.mean(x, axis=0)] * len(fields)
 
@@ -723,17 +736,20 @@ class Mesh3D(tfields.TensorMaps):
             mask = self.in_faces(tensor_field, delta=delta)
 
             face_indices = np.arange(n_faces)
-            point_face_assignment = [face_indices[mask[p_index, :]]
-                                     for p_index in point_indices]
-            point_face_assignment = np.array([fa if len(fa) != 0 else [-1]
-                                              for fa in point_face_assignment])
+            point_face_assignment = [
+                face_indices[mask[p_index, :]] for p_index in point_indices
+            ]
+            point_face_assignment = np.array(
+                [fa if len(fa) != 0 else [-1] for fa in point_face_assignment]
+            )
             point_face_assignment = point_face_assignment.reshape(-1)
         point_face_assignment_set = set(point_face_assignment)
 
         # merge the fields according to point_face_assignment
         map_fields = []
-        for field, map_field, merge_function in \
-                zip(fields, empty_map_fields, merge_functions):
+        for field, map_field, merge_function in zip(
+            fields, empty_map_fields, merge_functions
+        ):
             for i, f_index in enumerate(point_face_assignment_set):
                 if f_index == -1:
                     # point could not be mapped
@@ -761,10 +777,10 @@ class Mesh3D(tfields.TensorMaps):
 
         inst = self.copy()
 
-        '''
+        """
         add the indices of the vertices and maps to the fields. They will be
         removed afterwards
-        '''
+        """
         if not _in_recursion:
             inst.fields.append(tfields.Tensors(np.arange(len(inst))))
             for mp in inst.maps.values():
@@ -780,14 +796,14 @@ class Mesh3D(tfields.TensorMaps):
         elif all(~mask):
             # all vertices are valid
             inst = inst[mask]
-        elif at_intersection == 'keep':
+        elif at_intersection == "keep":
             expression_parts = tfields.lib.symbolics.split_expression(expression)
             if len(expression_parts) > 1:
                 new_mesh = inst.copy()
-                for exprPart in expression_parts:
-                    inst, _ = inst._cut_sympy(exprPart,
-                                              at_intersection=at_intersection,
-                                              _in_recursion=True)
+                for expr_part in expression_parts:
+                    inst, _ = inst._cut_sympy(
+                        expr_part, at_intersection=at_intersection, _in_recursion=True
+                    )
             elif len(expression_parts) == 1:
                 face_delete_indices = set([])
                 for i, face in enumerate(inst.maps[3]):
@@ -807,14 +823,14 @@ class Mesh3D(tfields.TensorMaps):
             else:
                 raise ValueError("Sympy expression is not splitable.")
             inst = inst.cleaned()
-        elif at_intersection == 'split' or at_intersection == 'splitRough':
-            '''
+        elif at_intersection == "split" or at_intersection == "split_rough":
+            """
             add vertices and faces that are at the border of the cuts
-            '''
+            """
             expression_parts = tfields.lib.symbolics.split_expression(expression)
             if len(expression_parts) > 1:
                 new_mesh = inst.copy()
-                if at_intersection == 'splitRough':
+                if at_intersection == "split_rough":
                     """
                     the following is, to speed up the process. Problem is, that
                     triangles can exist, where all points lie outside the cut,
@@ -825,45 +841,47 @@ class Mesh3D(tfields.TensorMaps):
                     face_inters_mask = np.full((inst.faces.shape[0]), False, dtype=bool)
                     for i, face in enumerate(inst.faces):
                         vertices_rejected = [-mask[f] for f in face]
-                        face_on_edge = any(vertices_rejected) and not all(vertices_rejected)
+                        face_on_edge = any(vertices_rejected) and not all(
+                            vertices_rejected
+                        )
                         if face_on_edge:
                             face_inters_mask[i] = True
                     new_mesh.remove_faces(-face_inters_mask)
 
-                for exprPart in expression_parts:
-                    inst, _ = inst._cut_sympy(exprPart,
-                                              at_intersection='split',
-                                              _in_recursion=True)
+                for expr_part in expression_parts:
+                    inst, _ = inst._cut_sympy(
+                        expr_part, at_intersection="split", _in_recursion=True
+                    )
             elif len(expression_parts) == 1:
-                points = [sympy.symbols('x0, y0, z0'),
-                          sympy.symbols('x1, y1, z1'),
-                          sympy.symbols('x2, y2, z2')]
+                # build plane from cut expression
                 plane_sympy = tfields.lib.symbolics.to_plane(expression)
                 norm_sympy = np.array(plane_sympy.normal_vector).astype(float)
                 d = -norm_sympy.dot(np.array(plane_sympy.p1).astype(float))
-                plane = {'normal': norm_sympy, 'd': d}
+                plane = {"normal": norm_sympy, "d": d}
 
+                # initialize empty containers
                 norm_vectors = inst.triangles().norms()
-                new_points = np.empty((0, 3))
+                new_vertices = np.empty((0, 3))
                 new_faces = np.empty((0, 3))
-                new_fields = [tfields.Tensors(np.empty((0,) + field.shape[1:]),
-                                              coord_sys=field.coord_sys)
-                              for field in inst.fields]
+                new_fields = [
+                    tfields.Tensors(
+                        np.empty((0,) + field.shape[1:]), coord_sys=field.coord_sys
+                    )
+                    for field in inst.fields
+                ]
                 new_map_fields = [[] for field in inst.maps[3].fields]
                 new_norm_vectors = []
-                newScalarMap = []
-                n_new = 0
 
+                # copy TODO?
                 vertices = np.array(inst)
                 faces = np.array(inst.maps[3])
                 fields = [np.array(field) for field in inst.fields]
-                faces_fields = [np.array(field)
-                                for field in inst.maps[3].fields]
+                faces_fields = [np.array(field) for field in inst.maps[3].fields]
 
-                face_delete_indices = set([])
+                face_delete_indices = set([])  # indexing not intersected faces
                 for i, face in enumerate(inst.maps[3]):
                     """
-                    vertices_rejected is a mask for each face that is True,
+                    vertices_rejected is a mask for each face that is True
                     where a point is on the rejected side of the plane
                     """
                     vertices_rejected = [~mask[f] for f in face]
@@ -880,8 +898,9 @@ class Mesh3D(tfields.TensorMaps):
                         Add the intersection points and faces
                         """
                         intersection = _intersect(
-                            triangle_points, plane, vertices_rejected)
-                        last_idx = len(vertices) - 1
+                            triangle_points, plane, vertices_rejected
+                        )
+                        last_idx = len(vertices)
                         for tri_list in intersection:
                             new_face = []
                             for item in tri_list:
@@ -896,28 +915,29 @@ class Mesh3D(tfields.TensorMaps):
                                     # new vertex
                                     new_face.append(len(vertices))
                                     vertices = np.append(
-                                        vertices,
-                                        [[float(x) for x in item]],
-                                        axis=0)
+                                        vertices, [[float(x) for x in item]], axis=0
+                                    )
                                     fields = [
-                                        np.append(field,
-                                                  np.full((1,) + field.shape[1:], np.nan),
-                                                  axis=0)
-                                        for field in fields]
+                                        np.append(
+                                            field,
+                                            np.full((1,) + field.shape[1:], np.nan),
+                                            axis=0,
+                                        )
+                                        for field in fields
+                                    ]
                             faces = np.append(faces, [new_face], axis=0)
-                            faces_fields = [np.append(field,
-                                                      [field[i]],
-                                                      axis=0)
-                                            for field in faces_fields]
+                            faces_fields = [
+                                np.append(field, [field[i]], axis=0)
+                                for field in faces_fields
+                            ]
                             faces_fields[-1][-1] = i
 
                 face_map = tfields.TensorFields(
-                    faces, *faces_fields,
-                    dtype=int,
-                    coord_sys=inst.maps[3].coord_sys)
-                inst = tfields.Mesh3D(vertices, *fields,
-                                      maps=[face_map],
-                                      coord_sys=inst.coord_sys)
+                    faces, *faces_fields, dtype=int, coord_sys=inst.maps[3].coord_sys
+                )
+                inst = tfields.Mesh3D(
+                    vertices, *fields, maps=[face_map], coord_sys=inst.coord_sys
+                )
                 mask = np.full(len(inst.maps[3]), True, dtype=bool)
                 for face_idx in range(len(inst.maps[3])):
                     if face_idx in face_delete_indices:
@@ -926,11 +946,13 @@ class Mesh3D(tfields.TensorMaps):
             else:
                 raise ValueError("Sympy expression is not splitable.")
             inst = inst.cleaned()
-        elif at_intersection == 'remove':
+        elif at_intersection == "remove":
             inst = inst[mask]
         else:
-            raise AttributeError("No at_intersection method called {at_intersection} "
-                                 "implemented".format(**locals()))
+            raise AttributeError(
+                "No at_intersection method called {at_intersection} "
+                "implemented".format(**locals())
+            )
 
         if _in_recursion:
             template = None
@@ -938,12 +960,11 @@ class Mesh3D(tfields.TensorMaps):
             template_field = inst.fields.pop(-1)
             template_maps = []
             for mp in inst.maps.values():
-                t_mp = tfields.TensorFields(tfields.Tensors(mp),
-                                            mp.fields.pop(-1))
+                t_mp = tfields.TensorFields(tfields.Tensors(mp), mp.fields.pop(-1))
                 template_maps.append(t_mp)
-            template = tfields.Mesh3D(tfields.Tensors(inst),
-                                      template_field,
-                                      maps=template_maps)
+            template = tfields.Mesh3D(
+                tfields.Tensors(inst), template_field, maps=template_maps
+            )
         return inst, template
 
     def _cut_template(self, template):
@@ -987,13 +1008,13 @@ class Mesh3D(tfields.TensorMaps):
         if template.fields:
             template_field = np.array(template.fields[0])
             if len(self) > 0:
-                '''
+                """
                 if new vertices have been created in the template, it is
                 in principle unclear what fields we have to refer to.
                 Thus in creating the template, we gave np.nan.
                 To make it fast, we replace nan with 0 as a dummy and correct
                 the field entries afterwards with np.nan.
-                '''
+                """
                 nan_mask = np.isnan(template_field)
                 template_field[nan_mask] = 0  # dummy reference to index 0.
                 template_field = template_field.astype(int)
@@ -1011,13 +1032,10 @@ class Mesh3D(tfields.TensorMaps):
                     mp_fields.append(field[0:0])  # np.empty
                 else:
                     mp_fields.append(field[template_mp.fields[0].astype(int)])
-            new_mp = tfields.TensorFields(tfields.Tensors(template_mp),
-                                          *mp_fields)
+            new_mp = tfields.TensorFields(tfields.Tensors(template_mp), *mp_fields)
             maps.append(new_mp)
 
-        inst = tfields.Mesh3D(tfields.Tensors(template),
-                              *fields,
-                              maps=maps)
+        inst = tfields.Mesh3D(tfields.Tensors(template), *fields, maps=maps)
         return inst
 
     def cut(self, *args, **kwargs):
@@ -1123,8 +1141,22 @@ class Mesh3D(tfields.TensorMaps):
         """
         Returns:
             disjoint_parts(List(cls)), templates(List(cls))
+
+            >>> import tfields
+            >>> a = tfields.Mesh3D(
+            ...     [[0, 0, 0], [1, 0, 0], [1, 1, 0], [0, 1, 0]],
+            ...     maps=[[[0, 1, 2], [0, 2, 3]]])
+            >>> b = a.copy()
+
+            >>> b[:, 0] += 2
+            >>> m = tfields.Mesh3D.merged(a, b)
+            >>> parts = m.disjoint_parts()
+            >>> aa, ba = parts
+            >>> assert aa.maps[3].equal(ba.maps[3])
+            >>> assert aa.equal(a)
+            >>> assert ba.equal(b)
         """
-        mp_description = self.disjoint_map(0)
+        mp_description = self.disjoint_map(3)
         parts = self.parts(mp_description)
         if not return_template:
             return parts
@@ -1132,25 +1164,27 @@ class Mesh3D(tfields.TensorMaps):
             templates = []
             for i, part in enumerate(parts):
                 template = part.copy()
-                template.maps[3].fields = [
-                    tfields.Tensors(mp_description[1][i])
-                ]
+                template.maps[3].fields = [tfields.Tensors(mp_description[1][i])]
                 templates.append(template)
             return parts, templates
 
     def plot(self, **kwargs):  # pragma: no cover
         """
-        Forwarding to plotTools.plot_mesh
+        Forwarding to rna.plotting.plot_mesh
         """
-        scalars_demanded = any([v in kwargs for v in ['vmin', 'vmax', 'cmap']])
-        map_index = kwargs.pop('map_index',
-                               None if not scalars_demanded else 0)
+        scalars_demanded = (
+            "color" not in kwargs
+            and "facecolors" not in kwargs
+            and any([v in kwargs for v in ["vmin", "vmax", "cmap"]])
+        )
+        map_index = kwargs.pop("map_index", None if not scalars_demanded else 0)
         if map_index is not None:
             if not len(self.maps[3]) == 0:
-                kwargs['color'] = self.maps[3].fields[map_index]
+                kwargs["color"] = self.maps[3].fields[map_index]
         return rna.plotting.plot_mesh(self, self.faces, **kwargs)
 
 
-if __name__ == '__main__':  # pragma: no cover
+if __name__ == "__main__":  # pragma: no cover
     import doctest
+
     doctest.testmod()
